- Model: CA - Percolation spread in a soil profile
--					STOCHASTIC INITIAL CONDITON: sandy bathymetry profile
--
-- Author: Daniel Machado & Yusi Li
--
-- Lesson: TerraME models MUST have the following structure
--              1) Define space
--				2) Define behavior
--				3) Define time / bath
--				4) Run simulations 
--

HOLLOW = 1
SPILL = 2
OIL = 3
SAND = 4
SPILL2 = 5

rand = Random()

counterb = 0
counterf = 0
countere = 0
count_t = 0
count_spill = 0

--------------------------------------------------------------
-- GLOBALS
--------------------------------------------------------------


rand2 = Random()

--------------------------------------------------------------
-- MODEL (1. space, 2. behavior, and 3. time)
--------------------------------------------------------------

-------------
--# SPACE #
-------------

world = CellularSpace{
	xdim = 100,
	ydim = 130
}




world:createNeighborhood{
	strategy = "moore",
	self = false
}

forEachCell(world, function(cell)
	if ( cell.y ~= 0 ) then 
		if ((( cell.x % 2)+  ( cell.y % 2))  % 2 ) == 0 then
	    	cell.cover = HOLLOW
			counterf =counterf + 1 -- not used
		else
	    	cell.cover = SAND
			countere = countere + 1 --not used
		end
	else
		cell.cover = SPILL
		count_spill = count_spill + 1
	end
end)

forEachCell(world, function(cell)
	if rand2:number() > 0.4 then 
		cell.prob = true
	else
		cell.prob = false
	end
end)

----------------
--# BEHAVIOR #
----------------


update = function(cs)
     still_fire = 0
	forEachCell(cs, function(cell)
		if cell.past.cover == HOLLOW then
			forEachNeighbor(cell, function(cell, neighbor)
			
			
		
			if neighbor.past.cover == SPILL then
				   if cell.prob == false or neighbor.y == cell.y + 1 then
				   cell.cover = HOLLOW 
				   else
				   cell.cover = SPILL
				   end
				end
			end)
		elseif cell.past.cover == SPILL then
			cell.cover = SPILL2
	
			
		elseif cell.past.cover == SPILL2 then
		cell.cover = OIL
			counterb= counterb + 1
		end
		
		
		if cell.cover == SPILL then
		still_fire = 1
		end
		
		end)
end

-----------
--# BATH #
-----------

t = Timer{
	Event{action = function(e)
		world:synchronize()
		update(world)
		
		if still_fire == 1 then
		count_t = count_t + 1
		end
	
		world:notify()
	end}
}

--------------------------------------------------------------
-- SIMULATION
--------------------------------------------------------------

legend = Legend {
	grouping = "uniquevalue",

	colorBar = {
		{value = HOLLOW, color = "orange"},
		{value = SPILL, color = "red"},
		{value = SPILL2, color = "blue"},
		{value = OIL, color = "black"},
		{value = SAND, color = "brown"}
	}
}

Observer{
	subject = world,
	attributes = {"cover"},
	legends = {legend}
}

world:notify()  

t:execute(150)

print("---------------------")
print("VOLUME", counterb - count_spill)
print("---------------------")
print("PATH", count_t)